_precomputed_chains = [
    [0, 1, -1, 2, -4, 8, -16, 32, -31],
    [0, 1, -1, 2, -3, 6, -12, 15, -30],
    [0, 1, -2, 4, -5, -8, 13, 16, -29],
    [0, 1, -1, -2, 3, 4, -7, 14, -28],
    [0, 1, -1, -2, 3, -3, 6, -12, 24, -27],
    [0, 1, -2, 4, -5, -8, 13, -26],
    [0, 1, -1, -2, 4, -8, 9, 16, -25],
    [0, 1, -1, -2, 3, -6, 12, -24],
    [0, 1, -2, 4, -8, 7, 16, -23],
    [0, 1, -1, -2, 4, -3, -8, 11, -22],
    [0, 1, -1, 2, -3, 6, -12, 15, -21],
    [0, 1, -2, 4, -5, 10, -20],
    [0, 1, -1, -2, 3, 4, -8, 16, -19],
    [0, 1, -1, -2, 4, -8, 9, -18],
    [0, 1, -2, 4, -8, 16, -17],
    [0, 1, -1, 2, -4, 8, -16],
    [0, 1, -1, -2, 3, -6, 12, -15],
    [0, 1, -2, 4, -5, 7, -14],
    [0, 1, -1, -2, 3, -6, 12, -13],
    [0, 1, -1, 2, -3, 6, -12],
    [0, 1, -2, 4, -5, 7, -11],
    [0, 1, -1, 2, -4, 5, -10],
    [0, 1, -1, 2, -4, 8, -9],
    [0, 1, -2, 4, -8],
    [0, 1, -1, -2, 3, 4, -7],
    [0, 1, -1, -2, 3, -6],
    [0, 1, -2, 4, -5],
    [0, 1, -1, 2, -4],
    [0, 1, -1, 2, -3],
    [0, 1, -2],
    [0, 1, -1],
    [0],
    [0, 1],
    [0, 1, -1, 2],
    [0, 1, -1, -2, 3],
    [0, 1, -2, 4],
    [0, 1, -1, 2, -4, 5],
    [0, 1, -1, 2, -3, 6],
    [0, 1, -2, 4, -5, 7],
    [0, 1, -1, 2, -4, 8],
    [0, 1, -1, -2, 4, -8, 9],
    [0, 1, -2, 4, -5, 10],
    [0, 1, -1, -2, 4, -3, -8, 11],
    [0, 1, -1, -2, 3, -6, 12],
    [0, 1, -2, 4, -5, -8, 13],
    [0, 1, -1, -2, 3, 4, -7, 14],
    [0, 1, -1, 2, -3, 6, -12, 15],
    [0, 1, -2, 4, -8, 16],
    [0, 1, -1, 2, -4, 8, -16, 17],
    [0, 1, -1, 2, -4, 8, -9, 18],
    [0, 1, -2, 4, -5, 7, -14, 19],
    [0, 1, -1, 2, -4, 5, -10, 20],
    [0, 1, -1, -2, 3, -6, 12, -15, 21],
    [0, 1, -2, 4, -5, 7, -11, 22],
    [0, 1, -1, -2, 3, -6, 12, -24, 23],
    [0, 1, -1, 2, -3, 6, -12, 24],
    [0, 1, -2, 4, -5, 10, -20, 25],
    [0, 1, -1, -2, 3, -6, 12, -13, 26],
    [0, 1, -1, -2, 3, 4, -7, 14, -28, 27],
    [0, 1, -2, 4, -5, 7, -14, 28],
    [0, 1, -1, -2, 3, 4, -7, 14, -28, 29],
    [0, 1, -1, -2, 3, -6, 12, -15, 30],
    [0, 1, -2, 4, -8, 16, -32, 31]
]


def _precomputed_chain(n):
    return copy(_precomputed_chains[n + 31])


def _append_if_new(L, n):
    # Probably, a speed-up trick can be applied here:
    if n not in L:  
        L.append(n)


def chain_with_computation(chain):
    """
    Given an addflip chain, show how numbers are obtained from previous ones.
    
    INPUT:
     - ``chain`` - list of integers [a_1, ... a_n] that is an addflip chain
    
    OUTPUT: 
     - list of tuples (x_i, y_i, a_i) summing to 0 with x[i] and y[i]
       appearing in ``chain`` prior to a_i, with the exception that the
       first two tupes are (0,) and (1,)
    """
    result = map(lambda x: (x,), chain[:2])
    for k in range(2, len(chain)):
        a = chain[k]
        if a == -2 * chain[k-1]:
            result.append((chain[k-1], chain[k-1], a))        
        else:
            for j in range(k-1, -1, -1):
                b = -a - chain[j]
                if b in chain[:j+1]:
                    result.append((b, chain[j], a))
                    break
    return result       
        

def addflip_chain(n):
    """
    Compute an addflip chain for the integer ``n``.
    
    INPUT: ``n`` - Integer
    
    OUTPUT: list of Integers
    
    An addflip chain is a sequence starting with 0 and 1 in which each
    further term a_k is -(a_i + a_j) for some i, j <= k.  The last term 
    of the sequence is n.
    
    EXAMPLES: TODO
    """
    # Implementation is iterative as a recursive implementation could segfault.
    subgoals = [n]
    chain = []
    while subgoals:
        # print "subgoals = %s" % subgoals
        n = subgoals.pop()
        if -31 <= n <= 31:
            for x in _precomputed_chain(n):
                _append_if_new(chain, x)
        elif 2.divides(n):
            k = -n // 2
            # The following speed-up trick can probably be improved:
            if (chain and k == chain[-1]) or k in chain:  
                _append_if_new(chain, n)
            else:
                subgoals.append(n)
                subgoals.append(k)
        else:
            k = n // 32
            m = 32 * k - n   
            quality_k = k.valuation(2) - len(_precomputed_chain(m)) 
            quality_k1 = (k+1).valuation(2) - len(_precomputed_chain(m+32)) 
            if quality_k1 > quality_k:
                m += 32
                k += 1 
            # The following speed-up trick can probably be improved:
            if (chain and k == chain[-1]) or k in chain:  
                for a in range(1, 6):
                    _append_if_new(chain, (-2)^a * k)
                if m in chain:
                    _append_if_new(chain, n)
                else:
                    subgoals.append(n)
                    subgoals.append(m)
            else:
                subgoals.append(n)
                subgoals.append(k)
    return chain
